@@ -1,5 +1,32 @@
-/* This include file declares all objects exported by the interpreter
- * kernel that may be used by extensions.
+/* extern.h: This include file declares all symbols exported by the interpreter
+ * kernel that may be used by applications or extensions.
+ *
+ * $Id: extern.h 243 2003-12-05 12:38:37Z sam $
+ *
+ * Copyright 1990, 1991, 1992, 1993, 1994, 1995, Oliver Laumann, Berlin
+ * Copyright 2002, 2003 Sam Hocevar <sam@zoy.org>, Paris
+ *
+ * This software was derived from Elk 1.2, which was Copyright 1987, 1988,
+ * 1989, Nixdorf Computer AG and TELES GmbH, Berlin (Elk 1.2 has been written
+ * by Oliver Laumann for TELES Telematic Services, Berlin, in a joint project
+ * between TELES and Nixdorf Microprocessor Engineering, Berlin).
+ *
+ * Oliver Laumann, TELES GmbH, Nixdorf Computer AG and Sam Hocevar, as co-
+ * owners or individual owners of copyright in this software, grant to any
+ * person or company a worldwide, royalty free, license to
+ *
+ *    i) copy this software,
+ *   ii) prepare derivative works based on this software,
+ *  iii) distribute copies of this software or derivative works,
+ *   iv) perform this software, or
+ *    v) display this software,
+ *
+ * provided that this notice is not removed and that neither Oliver Laumann
+ * nor Teles nor Nixdorf are deemed to have made any representations as to
+ * the suitability of this software for any purpose nor are held responsible
+ * for any defects of this software.
+ *
+ * THERE IS ABSOLUTELY NO WARRANTY FOR THIS SOFTWARE.
  */
 
 C_LINKAGE_BEGIN
@@ -11,7 +38,8 @@ extern Object P_Autoload P_((Object, Object));
 /* Bignums
  */
 extern Object Make_Uninitialized_Bignum P_((int));
-extern Bignum_Normalize_In_Place P_((struct S_Bignum *));
+extern void Bignum_Normalize_In_Place P_((struct S_Bignum *));
+extern double Bignum_To_Double P_((Object));
 
 /* Boolean operators
  */
@@ -30,30 +58,30 @@ extern Object Make_Char P_((int));
 extern Object P_Charp P_((Object));
 extern Object P_Char_Upcase P_((Object));
 extern Object P_Char_Downcase P_((Object));
-extern Object P_Chr_Eq P_((Object, Object));
-extern Object P_Chr_Less P_((Object, Object));
-extern Object P_Chr_Greater P_((Object, Object));
-extern Object P_Chr_Eq_Less P_((Object, Object));
-extern Object P_Chr_Eq_Greater P_((Object, Object));
-extern Object P_Chr_CI_Eq P_((Object, Object));
-extern Object P_Chr_CI_Less P_((Object, Object));
-extern Object P_Chr_CI_Greater P_((Object, Object));
-extern Object P_Chr_CI_Eq_Less P_((Object, Object));
-extern Object P_Chr_CI_Eq_Greater P_((Object, Object));
-extern Object P_Char_Upper_Case P_((Object));
-extern Object P_Char_Lower_Case P_((Object));
-extern Object P_Char_Alphabetic P_((Object));
-extern Object P_Char_Numeric P_((Object));
-extern Object P_Char_Whitespace P_((Object));
+extern Object P_Char_Eq P_((Object, Object));
+extern Object P_Char_Less P_((Object, Object));
+extern Object P_Char_Greater P_((Object, Object));
+extern Object P_Char_Eq_Less P_((Object, Object));
+extern Object P_Char_Eq_Greater P_((Object, Object));
+extern Object P_Char_CI_Eq P_((Object, Object));
+extern Object P_Char_CI_Less P_((Object, Object));
+extern Object P_Char_CI_Greater P_((Object, Object));
+extern Object P_Char_CI_Eq_Less P_((Object, Object));
+extern Object P_Char_CI_Eq_Greater P_((Object, Object));
+extern Object P_Char_Upper_Casep P_((Object));
+extern Object P_Char_Lower_Casep P_((Object));
+extern Object P_Char_Alphabeticp P_((Object));
+extern Object P_Char_Numericp P_((Object));
+extern Object P_Char_Whitespacep P_((Object));
 extern Object P_Char_To_Integer P_((Object));
 extern Object P_Integer_To_Char P_((Object));
 
 /* Continuations
  */
-extern Object P_Call_CC P_((Object));
+extern Object P_Call_With_Current_Continuation P_((Object));
 extern Object P_Dynamic_Wind P_((Object, Object, Object));
 extern Object P_Control_Pointp P_((Object));
-extern Object P_Control_Point_Env P_((Object));
+extern Object P_Control_Point_Environment P_((Object));
 
 /* Scheme strings --> C Strings
  */
@@ -70,12 +98,10 @@ extern Object P_Dump P_((Object));
 
 /* Lexical bindings, environments
  */
-extern Object Add_Binding P_((Object, Object, Object));
-extern Object Lookup_Symbol P_((Object, int));
 extern Object P_The_Environment P_((void));
 extern Object P_Define P_((Object));
 extern Object P_Set P_((Object));
-extern Object P_Env_List P_((Object));
+extern Object P_Environment_To_List P_((Object));
 extern Object P_Define_Macro P_((Object));
 extern Object P_Boundp P_((Object));
 extern Object P_Global_Environment P_((void));
@@ -84,12 +110,17 @@ extern Object The_Environment, Global_Environment;
 
 /* Error handling
  */
-extern Primitive_Error P_((ELLIPSIS));
-extern Fatal_Error P_((ELLIPSIS));
-extern Range_Error P_((Object));
-extern Panic P_((const char*));
+extern void Primitive_Error P_((const char*, ...)) elk_attribute(__noreturn__);
+extern void Fatal_Error P_((const char*, ...)) elk_attribute(__noreturn__);
+extern void Range_Error P_((Object));
+extern void Panic P_((const char*));
 extern Object P_Error P_((int, Object*));
 extern Object P_Reset P_((void));
+extern const char *Error_Tag;    /* will be removed in the near future */
+extern const char *Get_Error_Tag P_((void));
+extern void Set_Error_Tag P_((const char *));
+extern void Set_App_Name P_((char *));
+extern void Reader_Error P_((Object, char *)) elk_attribute(__noreturn__);
 
 /* Interrupts
  */
@@ -108,8 +139,8 @@ extern Object P_Require P_((int, Object*));
  */
 extern int GC_Debug;
 extern Object Alloc_Object P_((int, int, int));
-extern Register_Before_GC P_((void (*)(void)));
-extern Register_After_GC P_((void (*)(void)));
+extern void Register_Before_GC P_((void (*)(void)));
+extern void Register_After_GC P_((void (*)(void)));
 extern Object P_Collect P_((void));
 extern Object P_Garbage_Collect_Status P_((int, Object *));
 #ifdef GENERATIONAL_GC
@@ -120,19 +151,19 @@ extern Object P_Garbage_Collect_Status P_((int, Object *));
  */
 extern Object Curr_Input_Port, Curr_Output_Port;
 extern Object Standard_Input_Port, Standard_Output_Port;
-extern Reset_IO P_((int));
-extern Object P_Curr_Input_Port P_((void));
-extern Object P_Curr_Output_Port P_((void));
+extern void Reset_IO P_((int));
+extern Object P_Current_Input_Port P_((void));
+extern Object P_Current_Output_Port P_((void));
 extern Object P_Input_Portp P_((Object));
 extern Object P_Output_Portp P_((Object));
 extern Object P_Open_Input_File P_((Object));
 extern Object P_Open_Output_File P_((Object));
 extern Object P_Open_Input_Output_File P_((Object));
 extern Object P_Eof_Objectp P_((Object));
-extern Object P_With_Input P_((Object, Object));
-extern Object P_With_Output P_((Object, Object));
-extern Object P_Call_With_Input P_((Object, Object));
-extern Object P_Call_With_Output P_((Object, Object));
+extern Object P_With_Input_From_File P_((Object, Object));
+extern Object P_With_Output_To_File P_((Object, Object));
+extern Object P_Call_With_Input_File P_((Object, Object));
+extern Object P_Call_With_Output_File P_((Object, Object));
 extern Object P_Open_Input_String P_((Object));
 extern Object P_Open_Output_String P_((void));
 extern Object P_Port_File_Name P_((Object));
@@ -143,19 +174,21 @@ extern Object P_Close_Output_Port P_((Object));
 extern Object P_Port_Line_Number P_((Object));
 extern Object Terminate_File P_((Object));
 extern Object Make_Port P_((int, FILE*, Object));
-extern int Path_Max P_((void));
+extern unsigned int Path_Max P_((void));
 
 /* Loading of files
  */
 extern Object P_Load P_((int, Object*));
+extern void Load_Source_Port P_((Object));
+extern void Load_File P_((char *));
 
 /* Pairs and lists
  */
 extern Object P_Cons P_((Object, Object));
 extern Object P_Car P_((Object));
 extern Object P_Cdr P_((Object));
-extern Object P_Setcar P_((Object, Object));
-extern Object P_Setcdr P_((Object, Object));
+extern Object P_Set_Car P_((Object, Object));
+extern Object P_Set_Cdr P_((Object, Object));
 extern Object P_Listp P_((Object));
 extern Object P_List P_((int, Object*));
 extern Object P_Length P_((Object));
@@ -178,7 +211,7 @@ extern Object P_Caaaar P_((Object));
 extern Object P_Caaadr P_((Object));
 extern Object P_Caadar P_((Object));
 extern Object P_Caaddr P_((Object));
-extern Object P_Cadaar P_((Object)); 
+extern Object P_Cadaar P_((Object));
 extern Object P_Cadadr P_((Object));
 extern Object P_Caddar P_((Object));
 extern Object P_Cadddr P_((Object));
@@ -205,26 +238,27 @@ extern Object P_Memv P_((Object, Object));
 extern Object P_Member P_((Object, Object));
 extern Object P_Make_List P_((Object, Object));
 extern Object Copy_List P_((Object));
-extern int Fast_Length P_((Object));
+extern unsigned int Fast_Length P_((Object));
 extern Object Const_Cons P_((Object, Object));
 
 /* Startup and termination
  */
 extern Object P_Command_Line_Args P_((void));
 extern Object P_Exit P_((int, Object*));
+extern void Elk_Init P_((int, char **av, int, char *));
+extern void Exit_Handler P_((void));
 
 /* malloc() and realloc()
  */
-extern char *Safe_Malloc P_((unsigned));
-extern char *Safe_Realloc P_((char*, unsigned));
+extern char *Safe_Malloc P_((unsigned int));
+extern char *Safe_Realloc P_((char*, unsigned int));
 
 /* Numbers
  */
 extern Object Make_Integer P_((int));
-extern Object Make_Fixnum P_((int));
-extern Object Make_Unsigned P_((unsigned));
-extern Object Make_Long P_((long));
-extern Object Make_Unsigned_Long P_((unsigned long));
+extern Object Make_Unsigned P_((unsigned int));
+extern Object Make_Long P_((long int));
+extern Object Make_Unsigned_Long P_((unsigned long int));
 extern Object Make_Reduced_Flonum P_((double));
 extern Object Make_Flonum P_((double));
 extern Object P_Numberp P_((Object));
@@ -277,19 +311,23 @@ extern Object P_Random P_((void));
 extern Object P_Srandom P_((Object));
 extern Object P_Number_To_String P_((int, Object*));
 extern double Get_Double P_((Object));
-extern Get_Integer P_((Object));
-extern long Get_Long P_((Object));
-extern Get_Exact_Integer P_((Object));
-extern long Get_Exact_Long P_((Object));
+extern int Get_Integer P_((Object));
+extern unsigned int Get_Unsigned P_((Object));
+extern long int Get_Long P_((Object));
+extern unsigned long int Get_Unsigned_Long P_((Object));
+extern int Get_Exact_Integer P_((Object));
+extern unsigned int Get_Exact_Unsigned P_((Object));
+extern long int Get_Exact_Long P_((Object));
+extern unsigned long int Get_Exact_Unsigned_Long P_((Object));
 
 /* Onfork handlers
  */
-extern Register_Onfork P_((void (*)(void)));
+extern void Register_Onfork P_((void (*)(void)));
 extern void Call_Onfork P_((void));
 
 /* Define_Primitive()
  */
-extern Define_Primitive P_((Object (*)(ELLIPSIS), const char*, int, int,
+extern void Define_Primitive P_((Object (*)(ELLIPSIS), const char*, int, int,
     enum discipline));
 
 /* Output
@@ -303,23 +341,23 @@ extern Object P_Clear_Output_Port P_((int, Object*));
 extern Object P_Flush_Output_Port P_((int, Object*));
 extern Object P_Print P_((int, Object*));
 extern Object P_Get_Output_String P_((Object));
-extern Check_Output_Port P_((Object));
-extern Discard_Output P_((Object));
-extern Printf P_((ELLIPSIS));
-extern Print_Object P_((Object, Object, int, int, int));
-extern Format P_((Object, const char*, int, int, Object*));
+extern void Check_Output_Port P_((Object));
+extern void Discard_Output P_((Object));
+extern void Printf P_((Object, const char *, ...));
+extern void Print_Object P_((Object, Object, int, int, unsigned int));
+extern void General_Print_Object P_((Object, Object, int));
+extern void Format P_((Object, const char*, unsigned int, int, Object*));
 extern int Saved_Errno;
 
 /* Evaluator, procedures, macros
  */
-extern const char *Error_Tag;
 extern Object Eval P_((Object));
 extern Object P_Eval P_((int, Object*));
 extern Object P_Apply P_((int, Object*));
 extern Object Funcall P_((Object, Object, int));
 extern Object P_Lambda P_((Object));
 extern Object P_Map P_((int, Object*));
-extern Object P_Procedure_Env P_((Object));
+extern Object P_Procedure_Environment P_((Object));
 extern Object P_Procedure_Lambda P_((Object));
 extern Object P_For_Each P_((int, Object*));
 extern Object P_Procedurep P_((Object));
@@ -329,14 +367,14 @@ extern Object P_Macro_Expand P_((Object));
 extern Object P_Primitivep P_((Object));
 extern Object P_Compoundp P_((Object));
 extern Object P_Macrop P_((Object));
-extern Check_Procedure P_((Object));
+extern void Check_Procedure P_((Object));
 
 /* Delay and force
  */
 extern Object P_Delay P_((Object));
 extern Object P_Force P_((Object));
 extern Object P_Promisep P_((Object));
-extern Object P_Promise_Env P_((Object));
+extern Object P_Promise_Environment P_((Object));
 
 /* Input
  */
@@ -347,8 +385,9 @@ extern Object P_Char_Readyp P_((int, Object*));
 extern Object P_Unread_Char P_((int, Object*));
 extern Object P_Read_String P_((int, Object*));
 extern Object P_Clear_Input_Port P_((int, Object*));
-extern Check_Input_Port P_((Object));
-extern Discard_Input P_((Object));
+extern Object General_Read P_((Object, int));
+extern void Check_Input_Port P_((Object));
+extern void Discard_Input P_((Object));
 extern void Define_Reader P_((int, READFUN));
 
 /* Special forms
@@ -370,22 +409,22 @@ extern Object P_Begin1 P_((Object));
 
 /* Strings
  */
-extern Object Make_String P_((const char*, int));
-extern Object Make_Const_String P_((const char*, int));
+extern Object Make_String P_((const char*, unsigned int));
+extern Object Make_Const_String P_((const char*, unsigned int));
 extern Object P_Make_String P_((int, Object*));
 extern Object P_Stringp P_((Object));
 extern Object P_String P_((int, Object*));
 extern Object P_String_To_Number P_((int, Object*));
-extern Object P_Str_Eq P_((Object, Object));
-extern Object P_Str_Less P_((Object, Object));
-extern Object P_Str_Greater P_((Object, Object));
-extern Object P_Str_Eq_Less P_((Object, Object));
-extern Object P_Str_Eq_Greater P_((Object, Object));
-extern Object P_Str_CI_Eq P_((Object, Object));
-extern Object P_Str_CI_Less P_((Object, Object));
-extern Object P_Str_CI_Greater P_((Object, Object));
-extern Object P_Str_CI_Eq_Less P_((Object, Object));
-extern Object P_Str_CI_Eq_Greater P_((Object, Object));
+extern Object P_String_Eq P_((Object, Object));
+extern Object P_String_Less P_((Object, Object));
+extern Object P_String_Greater P_((Object, Object));
+extern Object P_String_Eq_Less P_((Object, Object));
+extern Object P_String_Eq_Greater P_((Object, Object));
+extern Object P_String_CI_Eq P_((Object, Object));
+extern Object P_String_CI_Less P_((Object, Object));
+extern Object P_String_CI_Greater P_((Object, Object));
+extern Object P_String_CI_Eq_Less P_((Object, Object));
+extern Object P_String_CI_Eq_Greater P_((Object, Object));
 extern Object P_String_Length P_((Object));
 extern Object P_String_Ref P_((Object, Object));
 extern Object P_String_Set P_((Object, Object, Object));
@@ -398,10 +437,18 @@ extern Object P_Substring_Fill P_((Object, Object, Object, Object));
 extern Object P_String_Fill P_((Object, Object));
 extern Object P_Substringp P_((Object, Object));
 extern Object P_CI_Substringp P_((Object, Object));
+extern int String_Getc P_((Object));
+extern void String_Ungetc P_((Object, register int));
 
 /* Symbols, variables, frequently used Scheme objects
  */
-extern Object Null, True, False, Void, Newline, Eof;
+extern_c Object elk_import Null;
+extern_c Object elk_import True;
+extern_c Object elk_import False;
+extern_c Object elk_import Void;
+extern_c Object elk_import Newline;
+extern_c Object elk_import Eof;
+extern_c Object elk_import Unspecified;
 extern Object Intern P_((const char*));
 extern Object CI_Intern P_((const char*));
 extern Object P_Oblist P_((void));
@@ -413,34 +460,37 @@ extern Object P_Get P_((Object, Object));
 extern Object P_Symbol_Plist P_((Object));
 extern void Define_Variable P_((Object*, const char*, Object));
 extern void Define_Symbol P_((Object *, const char*));
+extern Object Sym_Else;
 extern Object Var_Get P_((Object));
 extern void Var_Set P_((Object, Object));
-extern unsigned long Symbols_To_Bits P_((Object, int, SYMDESCR*));
-extern Object Bits_To_Symbols P_((unsigned long, int, SYMDESCR*));
+extern int Var_Is_True P_((Object));
+extern unsigned long int Symbols_To_Bits P_((Object, int, SYMDESCR*));
+extern Object Bits_To_Symbols P_((unsigned long int, int, SYMDESCR*));
 
 /* Termination functions
  */
 extern void Register_Object P_((Object, GENERIC, PFO, int));
 extern void Deregister_Object P_((Object));
-extern Object Find_Object P_((ELLIPSIS));
+extern Object Find_Object P_((int, GENERIC, MATCHFUN, ...));
 extern void Terminate_Group P_((GENERIC));
 extern void Terminate_Type P_((int));
 
 /* Types, Define_Type()
  */
-extern TYPEDESCR Types[];
+extern TYPEDESCR *Types;
 extern Object P_Type P_((Object));
-extern Wrong_Type P_((Object, int));
-extern Wrong_Type_Combination P_((Object, const char*));
-extern Define_Type P_((int, const char*, int (*)(Object), int,
+extern void Wrong_Type P_((Object, int)) elk_attribute(__noreturn__);
+extern void Wrong_Type_Combination P_((Object, const char*))
+    elk_attribute(__noreturn__);
+extern int Define_Type P_((int, const char*, int (*)(Object), int,
     int (*)(Object, Object), int (*)(Object, Object),
     int (*)(Object, Object, int, int, int),
     int (*)(Object*, int (*)(Object*)) ));
 
 /* Vectors
  */
-extern Object Make_Vector P_((int, Object));
-extern Object Make_Const_Vector P_((int, Object));
+extern Object Make_Vector P_((unsigned int, Object));
+extern Object Make_Const_Vector P_((unsigned int, Object));
 extern Object P_Make_Vector P_((int, Object*));
 extern Object P_Vectorp P_((Object));
 extern Object P_Vector P_((int, Object*));
@@ -451,6 +501,5 @@ extern Object P_Vector_To_List P_((Object));
 extern Object P_List_To_Vector P_((Object));
 extern Object P_Vector_Fill P_((Object, Object));
 extern Object P_Vector_Copy P_((Object));
-extern Object List_To_Vector P_((Object, int));
 
 C_LINKAGE_END
