@@ -1,8 +1,35 @@
-/* Elk/GDBM-interface.
- *
+/* gdbm.c: Elk/GDBM-interface.
  * Original version by Martin Stut <stut@informatik.tu-muenchen.dbp.de>.
  *
- * Functions exported:
+ * $Id: gdbm.c 243 2003-12-05 12:38:37Z sam $
+ *
+ * Copyright 1990, 1991, 1992, 1993, 1994, 1995, Oliver Laumann, Berlin
+ * Copyright 2002, 2003 Sam Hocevar <sam@zoy.org>, Paris
+ *
+ * This software was derived from Elk 1.2, which was Copyright 1987, 1988,
+ * 1989, Nixdorf Computer AG and TELES GmbH, Berlin (Elk 1.2 has been written
+ * by Oliver Laumann for TELES Telematic Services, Berlin, in a joint project
+ * between TELES and Nixdorf Microprocessor Engineering, Berlin).
+ *
+ * Oliver Laumann, TELES GmbH, Nixdorf Computer AG and Sam Hocevar, as co-
+ * owners or individual owners of copyright in this software, grant to any
+ * person or company a worldwide, royalty free, license to
+ *
+ *    i) copy this software,
+ *   ii) prepare derivative works based on this software,
+ *  iii) distribute copies of this software or derivative works,
+ *   iv) perform this software, or
+ *    v) display this software,
+ *
+ * provided that this notice is not removed and that neither Oliver Laumann
+ * nor Teles nor Nixdorf are deemed to have made any representations as to
+ * the suitability of this software for any purpose nor are held responsible
+ * for any defects of this software.
+ *
+ * THERE IS ABSOLUTELY NO WARRANTY FOR THIS SOFTWARE.
+ */
+
+/* Functions exported:
  *
  * (gdbm-file? obj)
  *
@@ -66,9 +93,13 @@
  */
 
 
-#include "scheme.h"
+#include "config.h"
+
 #include <gdbm.h>
 #include <errno.h>
+#include <string.h>
+
+#include "scheme.h"
 
 extern gdbm_error gdbm_errno;
 extern int errno;
@@ -98,37 +129,37 @@ struct S_gdbm_fh{
 
 #define GDBM_FH(obj) ((struct S_gdbm_fh *)POINTER(obj))
 
-int Gdbm_fh_Equal (a, b) Object a, b; {
+int Gdbm_fh_Equal (Object a, Object b) {
     return !GDBM_FH(a)->free && !GDBM_FH(b)->free &&
-	    GDBM_FH(a)->fptr == GDBM_FH(b)->fptr;
+            GDBM_FH(a)->fptr == GDBM_FH(b)->fptr;
 }
 
 /*ARGSUSED*/
-Gdbm_fh_Print (fh, port, raw, depth, len) Object fh, port;
-	int /*Bool*/ raw; int depth, len; {
+int Gdbm_fh_Print (Object fh, Object port, int raw, int depth, int len) {
     Printf (port, "#[gdbm-file %lu]", GDBM_FH(fh)->fptr);
+    return 0;
 }
 
-Object P_Gdbm_filep (x) Object x; {
+Object P_Gdbm_filep (Object x) {
     return TYPE(x) == T_Gdbm_fh ? True : False;
 }
 
-static void Fatal_Func (s) char *s; {
+static void Fatal_Func (char *s) {
     gdbm_error_message = s;
     fprintf (stderr, "gdbm error: %s\n", s);
 }
 
-Object P_Gdbm_Open (argc, argv) Object *argv; {
+Object P_Gdbm_Open (int argc, Object *argv) {
     Object Gdbm_fh;
     GDBM_FILE dbf;
 
     Disable_Interrupts;
     dbf = gdbm_open (Get_Strsym (argv[0]), Get_Integer (argv[1]),
-	Symbols_To_Bits (argv[2], 0, RW_Syms),
-	argc == 4 ? Get_Integer (argv[3]) : 0644, Fatal_Func);
+        Symbols_To_Bits (argv[2], 0, RW_Syms),
+        argc == 4 ? Get_Integer (argv[3]) : 0644, Fatal_Func);
     if (dbf == 0) {
-	Enable_Interrupts;
-	return False;
+        Enable_Interrupts;
+        return False;
     }
     Gdbm_fh = Alloc_Object (sizeof (struct S_gdbm_fh), T_Gdbm_fh, 0);
     GDBM_FH (Gdbm_fh)->tag = Null;
@@ -138,13 +169,13 @@ Object P_Gdbm_Open (argc, argv) Object *argv; {
     return Gdbm_fh;
 }
 
-GDBM_FILE Check_Fh (fh) Object fh; {
+void Check_Fh (Object fh) {
     Check_Type (fh, T_Gdbm_fh);
     if (GDBM_FH(fh)->free)
-	Primitive_Error ("invalid gdbm-file: ~s", fh);
+        Primitive_Error ("invalid gdbm-file: ~s", fh);
 }
 
-Object P_Gdbm_Close (fh) Object fh; {
+Object P_Gdbm_Close (Object fh) {
     Check_Fh (fh);
     GDBM_FH(fh)->free = 1;
     Disable_Interrupts;
@@ -153,8 +184,7 @@ Object P_Gdbm_Close (fh) Object fh; {
     return Void;
 }
 
-Object P_Gdbm_Store (fh, key, content, flag)
-	Object fh, key, content, flag; {
+Object P_Gdbm_Store (Object fh, Object key, Object content, Object flag) {
     int res;
     datum k, c;
 
@@ -167,12 +197,12 @@ Object P_Gdbm_Store (fh, key, content, flag)
     c.dsize = STRING(content)->size;
     Disable_Interrupts;
     res = gdbm_store (GDBM_FH(fh)->fptr, k, c,
-	Symbols_To_Bits (flag, 0, Flag_Syms));
+        Symbols_To_Bits (flag, 0, Flag_Syms));
     Enable_Interrupts;
     return Make_Integer (res);
 }
 
-static Object Gdbm_Get (fh, key, func) Object fh, key; datum (*func)(); {
+static Object Gdbm_Get (Object fh, Object key, datum (*func)()) {
     Object res;
     datum k, c;
 
@@ -184,21 +214,21 @@ static Object Gdbm_Get (fh, key, func) Object fh, key; datum (*func)(); {
     c = (*func) (GDBM_FH(fh)->fptr, k);
     Enable_Interrupts;
     if (c.dptr == 0)
-	return False;
+        return False;
     res = Make_String (c.dptr, c.dsize);
     free (c.dptr);
     return res;
 }
 
-Object P_Gdbm_Fetch (fh, key) Object fh, key; {
+Object P_Gdbm_Fetch (Object fh, Object key) {
     return Gdbm_Get (fh, key, gdbm_fetch);
 }
 
-Object P_Gdbm_Nextkey (fh, key) Object fh, key; {
+Object P_Gdbm_Nextkey (Object fh, Object key) {
     return Gdbm_Get (fh, key, gdbm_nextkey);
 }
 
-Object P_Gdbm_Delete (fh, key) Object fh, key; {
+Object P_Gdbm_Delete (Object fh, Object key) {
     int res;
     datum k;
 
@@ -212,7 +242,7 @@ Object P_Gdbm_Delete (fh, key) Object fh, key; {
     return res == 0 ? True : False;
 }
 
-Object P_Gdbm_Firstkey (fh) Object fh; {
+Object P_Gdbm_Firstkey (Object fh) {
     Object res;
     datum k;
 
@@ -220,14 +250,14 @@ Object P_Gdbm_Firstkey (fh) Object fh; {
     Disable_Interrupts;
     k = gdbm_firstkey (GDBM_FH(fh)->fptr);
     Enable_Interrupts;
-    if (k.dptr == 0) 
-	return False;
+    if (k.dptr == 0)
+        return False;
     res = Make_String (k.dptr, k.dsize);
     free (k.dptr);
     return res;
 }
 
-Object P_Gdbm_Reorganize (fh) Object fh; {
+Object P_Gdbm_Reorganize (Object fh) {
     Check_Fh (fh);
     Disable_Interrupts;
     gdbm_reorganize (GDBM_FH(fh)->fptr);
@@ -243,7 +273,7 @@ Object P_Gdbm_Error_Text () {
     return Make_String (gdbm_error_message, strlen (gdbm_error_message));
 }
 
-elk_init_lib_gdbm () {
+void elk_init_lib_gdbm () {
     Define_Primitive (P_Gdbm_Open, "gdbm-open", 3, 4, VARARGS);
     Define_Primitive (P_Gdbm_filep, "gdbm-file?", 1, 1, EVAL);
     Define_Primitive (P_Gdbm_Close, "gdbm-close", 1, 1, EVAL);
@@ -256,7 +286,7 @@ elk_init_lib_gdbm () {
     Define_Primitive (P_Gdbm_Error, "gdbm-error", 0, 0, EVAL);
     Define_Primitive (P_Gdbm_Error_Text, "gdbm-error-text", 0, 0, EVAL);
     T_Gdbm_fh = Define_Type (0, "gdbm-file", NOFUNC,
-	sizeof (struct S_gdbm_fh), Gdbm_fh_Equal, Gdbm_fh_Equal,
-	Gdbm_fh_Print, NOFUNC);
-    P_Provide (Intern ("gdbm.o"));
+        sizeof (struct S_gdbm_fh), Gdbm_fh_Equal, Gdbm_fh_Equal,
+        Gdbm_fh_Print, NOFUNC);
+    P_Provide (Intern ("gdbm.la"));
 }
