@@ -1,6 +1,36 @@
+/* type.c
+ *
+ * $Id: type.c 243 2003-12-05 12:38:37Z sam $
+ *
+ * Copyright 1990, 1991, 1992, 1993, 1994, 1995, Oliver Laumann, Berlin
+ * Copyright 2002, 2003 Sam Hocevar <sam@zoy.org>, Paris
+ *
+ * This software was derived from Elk 1.2, which was Copyright 1987, 1988,
+ * 1989, Nixdorf Computer AG and TELES GmbH, Berlin (Elk 1.2 has been written
+ * by Oliver Laumann for TELES Telematic Services, Berlin, in a joint project
+ * between TELES and Nixdorf Microprocessor Engineering, Berlin).
+ *
+ * Oliver Laumann, TELES GmbH, Nixdorf Computer AG and Sam Hocevar, as co-
+ * owners or individual owners of copyright in this software, grant to any
+ * person or company a worldwide, royalty free, license to
+ *
+ *    i) copy this software,
+ *   ii) prepare derivative works based on this software,
+ *  iii) distribute copies of this software or derivative works,
+ *   iv) perform this software, or
+ *    v) display this software,
+ *
+ * provided that this notice is not removed and that neither Oliver Laumann
+ * nor Teles nor Nixdorf are deemed to have made any representations as to
+ * the suitability of this software for any purpose nor are held responsible
+ * for any defects of this software.
+ *
+ * THERE IS ABSOLUTELY NO WARRANTY FOR THIS SOFTWARE.
+ */
+
 #include "xlib.h"
 
-static Object Set_Attr_Slots; 
+static Object Set_Attr_Slots;
 static Object Conf_Slots;
 static Object GC_Slots;
 static Object Geometry_Slots;
@@ -15,35 +45,35 @@ static Object Sym_Parent_Relative, Sym_Copy_From_Parent;
 XSetWindowAttributes SWA;
 RECORD Set_Attr_Rec[] = {
     { (char *)&SWA.background_pixmap,     "background-pixmap",     T_BACKGROUND,
-	0,                  CWBackPixmap },
+        0,                  CWBackPixmap },
     { (char *)&SWA.background_pixel,      "background-pixel",      T_PIXEL,
-	0,                  CWBackPixel },
+        0,                  CWBackPixel },
     { (char *)&SWA.border_pixmap,         "border-pixmap",         T_BORDER,
-	0,                  CWBorderPixmap },
+        0,                  CWBorderPixmap },
     { (char *)&SWA.border_pixel,          "border-pixel",          T_PIXEL,
-	0,                  CWBorderPixel },
+        0,                  CWBorderPixel },
     { (char *)&SWA.bit_gravity,           "bit-gravity",           T_SYM,
-	Bit_Grav_Syms,      CWBitGravity },
+        Bit_Grav_Syms,      CWBitGravity },
     { (char *)&SWA.win_gravity,           "gravity",               T_SYM,
-	Grav_Syms,          CWWinGravity },
+        Grav_Syms,          CWWinGravity },
     { (char *)&SWA.backing_store,         "backing-store",         T_SYM,
-	Backing_Store_Syms, CWBackingStore },
+        Backing_Store_Syms, CWBackingStore },
     { (char *)&SWA.backing_planes,        "backing-planes",        T_PIXEL,
-	0,                  CWBackingPlanes },
+        0,                  CWBackingPlanes },
     { (char *)&SWA.backing_pixel,         "backing-pixel",         T_PIXEL,
-	0,                  CWBackingPixel },
+        0,                  CWBackingPixel },
     { (char *)&SWA.save_under,            "save-under",            T_BOOL,
-	0,                  CWSaveUnder },
+        0,                  CWSaveUnder },
     { (char *)&SWA.event_mask,            "event-mask",            T_MASK,
-	Event_Syms,         CWEventMask },
+        Event_Syms,         CWEventMask },
     { (char *)&SWA.do_not_propagate_mask, "do-not-propagate-mask", T_MASK,
-	Event_Syms,         CWDontPropagate },
+        Event_Syms,         CWDontPropagate },
     { (char *)&SWA.override_redirect,     "override-redirect",     T_BOOL,
-	0,                  CWOverrideRedirect },
+        0,                  CWOverrideRedirect },
     { (char *)&SWA.colormap,              "colormap",              T_COLORMAP,
-	0,                  CWColormap },
+        0,                  CWColormap },
     { (char *)&SWA.cursor,                "cursor",                T_CURSOR,
-	0,                  CWCursor },
+        0,                  CWCursor },
     { 0, 0, T_NONE, 0, 0 }
 };
 int Set_Attr_Size = sizeof Set_Attr_Rec / sizeof (RECORD);
@@ -57,7 +87,7 @@ RECORD Conf_Rec[] = {
     { (char *)&WC.border_width, "border-width", T_INT,     0, CWBorderWidth },
     { (char *)&WC.sibling,      "sibling",      T_WINDOW,  0, CWSibling },
     { (char *)&WC.stack_mode,   "stack-mode",   T_SYM,     Stack_Mode_Syms,
-	CWStackMode },
+        CWStackMode },
     { 0, 0, T_NONE, 0, 0 }
 };
 int Conf_Size = sizeof Conf_Rec / sizeof (RECORD);
@@ -65,51 +95,51 @@ int Conf_Size = sizeof Conf_Rec / sizeof (RECORD);
 XGCValues GCV;
 RECORD GC_Rec[] = {
     { (char *)&GCV.function,           "function",       T_SYM,
-	Func_Syms,        GCFunction },
+        Func_Syms,        GCFunction },
     { (char *)&GCV.plane_mask,         "plane-mask",     T_PIXEL,
-	0,                GCPlaneMask },
+        0,                GCPlaneMask },
     { (char *)&GCV.foreground,         "foreground",     T_PIXEL,
-	0,                GCForeground },
+        0,                GCForeground },
     { (char *)&GCV.background,         "background",     T_PIXEL,
-	0,                GCBackground },
+        0,                GCBackground },
     { (char *)&GCV.line_width,         "line-width",     T_INT,
-	0,                GCLineWidth },
+        0,                GCLineWidth },
     { (char *)&GCV.line_style,         "line-style",     T_SYM,
-	Line_Style_Syms,  GCLineStyle },
+        Line_Style_Syms,  GCLineStyle },
     { (char *)&GCV.cap_style,          "cap-style",      T_SYM,
-	Cap_Style_Syms,   GCCapStyle },
+        Cap_Style_Syms,   GCCapStyle },
     { (char *)&GCV.join_style,         "join-style",     T_SYM,
-	Join_Style_Syms,  GCJoinStyle },
+        Join_Style_Syms,  GCJoinStyle },
     { (char *)&GCV.fill_style,         "fill-style",     T_SYM,
-	Fill_Style_Syms,  GCFillStyle },
+        Fill_Style_Syms,  GCFillStyle },
     { (char *)&GCV.fill_rule,          "fill-rule",      T_SYM,
-	Fill_Rule_Syms,   GCFillRule },
+        Fill_Rule_Syms,   GCFillRule },
     { (char *)&GCV.arc_mode,           "arc-mode",       T_SYM,
-	Arc_Mode_Syms,    GCArcMode },
+        Arc_Mode_Syms,    GCArcMode },
     { (char *)&GCV.tile,               "tile",           T_PIXMAP,
-	0,                GCTile },
+        0,                GCTile },
     { (char *)&GCV.stipple,            "stipple",        T_PIXMAP,
-	0,                GCStipple },
+        0,                GCStipple },
     { (char *)&GCV.ts_x_origin,        "ts-x",           T_INT,
-	0,                GCTileStipXOrigin },
+        0,                GCTileStipXOrigin },
     { (char *)&GCV.ts_y_origin,        "ts-y",           T_INT,
-	0,                GCTileStipYOrigin },
+        0,                GCTileStipYOrigin },
     { (char *)&GCV.font,               "font",           T_FONT,
-	0,                GCFont },
+        0,                GCFont },
     { (char *)&GCV.subwindow_mode,     "subwindow-mode", T_SYM,
-	Subwin_Mode_Syms, GCSubwindowMode },
+        Subwin_Mode_Syms, GCSubwindowMode },
     { (char *)&GCV.graphics_exposures, "exposures",      T_BOOL,
-	0,                GCGraphicsExposures },
+        0,                GCGraphicsExposures },
     { (char *)&GCV.clip_x_origin,      "clip-x",         T_INT,
-	0,                GCClipXOrigin },
+        0,                GCClipXOrigin },
     { (char *)&GCV.clip_y_origin,      "clip-y",         T_INT,
-	0,                GCClipYOrigin },
+        0,                GCClipYOrigin },
     { (char *)&GCV.clip_mask,          "clip-mask",      T_PIXMAP,
-	0,                GCClipMask },
+        0,                GCClipMask },
     { (char *)&GCV.dash_offset,        "dash-offset",    T_INT,
-	0,                GCDashOffset },
+        0,                GCDashOffset },
     { (char *)&GCV.dashes,             "dashes",         T_CHAR,
-	0,                GCDashList },
+        0,                GCDashList },
     {0, 0, T_NONE, 0, 0 }
 };
 int GC_Size = sizeof GC_Rec / sizeof (RECORD);
@@ -130,55 +160,55 @@ int Geometry_Size = sizeof Geometry_Rec / sizeof (RECORD);
 XWindowAttributes WA;
 RECORD Win_Attr_Rec[] = {
     { (char *)&WA.x,                    "x",                      T_INT,
-	0,                  0 },
+        0,                  0 },
     { (char *)&WA.y,                    "y",                      T_INT,
-	0,                  0 },
+        0,                  0 },
     { (char *)&WA.width,                "width",                  T_INT,
-	0,                  0 },
+        0,                  0 },
     { (char *)&WA.height,               "height",                 T_INT,
-	0,                  0 },
+        0,                  0 },
     { (char *)&WA.border_width,         "border-width",           T_INT,
-	0,                  0 },
+        0,                  0 },
     { (char *)&WA.depth,                "depth",                  T_INT,
-	0,                  0 },
+        0,                  0 },
     { (char *)&WA.visual,               "visual",                 T_NONE,
-	0,                  0 },
+        0,                  0 },
     { (char *)&WA.root,                 "root",                   T_WINDOW,
-	0,                  0 },
+        0,                  0 },
 #if defined(__cplusplus) || defined(c_plusplus)
     { (char *)&WA.c_class,              "class",                  T_SYM,
 #else
     { (char *)&WA.class,                "class",                  T_SYM,
 #endif
-	Class_Syms,         0 },
+        Class_Syms,         0 },
     { (char *)&WA.bit_gravity,          "bit-gravity",            T_SYM,
-	Bit_Grav_Syms,      0 },
+        Bit_Grav_Syms,      0 },
     { (char *)&WA.win_gravity,          "gravity",                T_SYM,
-	Grav_Syms,          0 },
+        Grav_Syms,          0 },
     { (char *)&WA.backing_store,        "backing-store",          T_SYM,
-	Backing_Store_Syms, 0 },
+        Backing_Store_Syms, 0 },
     { (char *)&WA.backing_planes,       "backing-planes",         T_PIXEL,
-	0,                  0 },
+        0,                  0 },
     { (char *)&WA.backing_pixel,        "backing-pixel",          T_PIXEL,
-	0,                  0 },
+        0,                  0 },
     { (char *)&WA.save_under,           "save-under",             T_BOOL,
-	0,                  0 },
+        0,                  0 },
     { (char *)&WA.colormap ,            "colormap",               T_COLORMAP,
-	0,                  0 },
+        0,                  0 },
     { (char *)&WA.map_installed,        "map-installed",          T_BOOL,
-	0,                  0 },
+        0,                  0 },
     { (char *)&WA.map_state,            "map-state",              T_SYM,
-	Map_State_Syms,     0 },
+        Map_State_Syms,     0 },
     { (char *)&WA.all_event_masks,      "all-event-masks",        T_MASK,
-	Event_Syms,         0 },
+        Event_Syms,         0 },
     { (char *)&WA.your_event_mask,      "your-event-mask",        T_MASK,
-	Event_Syms,         0 },
+        Event_Syms,         0 },
     { (char *)&WA.do_not_propagate_mask, "do-not-propagate-mask", T_MASK,
-	Event_Syms,         0 },
+        Event_Syms,         0 },
     { (char *)&WA.override_redirect,    "override-redirect",      T_BOOL,
-	0,                  0 },
+        0,                  0 },
     { (char *)&WA.screen,               "screen",                 T_NONE,
-	0,                  0 },
+        0,                  0 },
     {0, 0, T_NONE, 0, 0 }
 };
 int Win_Attr_Size = sizeof Win_Attr_Rec / sizeof (RECORD);
@@ -186,23 +216,23 @@ int Win_Attr_Size = sizeof Win_Attr_Rec / sizeof (RECORD);
 XFontStruct FI;
 RECORD Font_Info_Rec[] = {
     { (char *)&FI.direction,            "direction",             T_SYM,
-	Direction_Syms,     0 },
+        Direction_Syms,     0 },
     { (char *)&FI.min_char_or_byte2,    "min-byte2",             T_INT,
-	0,                  0 },
+        0,                  0 },
     { (char *)&FI.max_char_or_byte2,    "max-byte2",             T_INT,
-	0,                  0 },
+        0,                  0 },
     { (char *)&FI.min_byte1,            "min-byte1",             T_INT,
-	0,                  0 },
+        0,                  0 },
     { (char *)&FI.max_byte1,            "max-byte1",             T_INT,
-	0,                  0 },
+        0,                  0 },
     { (char *)&FI.all_chars_exist,      "all-chars-exist?",      T_BOOL,
-	0,                  0 },
+        0,                  0 },
     { (char *)&FI.default_char,         "default-char",          T_INT,
-	0,                  0 },
+        0,                  0 },
     { (char *)&FI.ascent,               "ascent",                T_INT,
-	0,                  0 },
+        0,                  0 },
     { (char *)&FI.descent,              "descent",               T_INT,
-	0,                  0 },
+        0,                  0 },
     {0, 0, T_NONE, 0, 0 }
 };
 int Font_Info_Size = sizeof Font_Info_Rec / sizeof (RECORD);
@@ -222,21 +252,21 @@ int Char_Info_Size = sizeof Char_Info_Rec / sizeof (RECORD);
 XWMHints WMH;
 RECORD Wm_Hints_Rec[] = {
     { (char *)&WMH.input,         "input?",        T_BOOL,
-	0,                  InputHint },
+        0,                  InputHint },
     { (char *)&WMH.initial_state, "initial-state", T_SYM,
-	Initial_State_Syms, StateHint },
+        Initial_State_Syms, StateHint },
     { (char *)&WMH.icon_pixmap,   "icon-pixmap",   T_PIXMAP,
-	0,                  IconPixmapHint },
+        0,                  IconPixmapHint },
     { (char *)&WMH.icon_window,   "icon-window",   T_WINDOW,
-	0,                  IconWindowHint },
+        0,                  IconWindowHint },
     { (char *)&WMH.icon_x,        "icon-x",        T_INT,
-	0,                  IconPositionHint },
+        0,                  IconPositionHint },
     { (char *)&WMH.icon_y,        "icon-y",        T_INT,
-	0,                  IconPositionHint },
+        0,                  IconPositionHint },
     { (char *)&WMH.icon_mask,     "icon-mask",     T_PIXMAP,
-	0,                  IconMaskHint },
+        0,                  IconMaskHint },
     { (char *)&WMH.window_group,  "window-group",  T_WINDOW,
-	0,                  WindowGroupHint },
+        0,                  WindowGroupHint },
     {0, 0, T_NONE, 0, 0 }
 };
 int Wm_Hints_Size = sizeof Wm_Hints_Rec / sizeof (RECORD);
@@ -263,81 +293,81 @@ RECORD Size_Hints_Rec[] = {
     { (char *)&SZH.max_aspect.y,   "max-aspect-y",    T_INT,  0, PAspect },
     { (char *)&SZH.base_width,     "base-width",      T_INT,  0, PBaseSize },
     { (char *)&SZH.base_height,    "base-height",     T_INT,  0, PBaseSize },
-    { (char *)&SZH.win_gravity,    "gravity",         T_SYM,  Grav_Syms, 
-								 PWinGravity },
+    { (char *)&SZH.win_gravity,    "gravity",         T_SYM,  Grav_Syms,
+                                                                 PWinGravity },
     {0, 0, T_NONE, 0, 0 }
 };
 int Size_Hints_Size = sizeof Size_Hints_Rec / sizeof (RECORD);
 
-unsigned long Vector_To_Record (v, len, sym, rp) Object v, sym;
-	register RECORD *rp; {
+unsigned long Vector_To_Record (Object v, unsigned int len, Object sym,
+                                register RECORD *rp) {
     register Object *p;
     unsigned long mask = 0;
 
     Check_Type (v, T_Vector);
     p = VECTOR(v)->data;
     if (VECTOR(v)->size != len && !EQ(p[0], sym))
-	Primitive_Error ("invalid argument");
+        Primitive_Error ("invalid argument");
     for ( ; rp->slot; rp++) {
-	++p;
-	if (rp->type == T_NONE || Nullp (*p))
-	    continue;
-	switch (rp->type) {
-	case T_INT:
-	    *(int *)rp->slot = Get_Integer (*p); break;
-	case T_SHORT:
-	    *(short *)rp->slot = Get_Integer (*p); break;
-	case T_CHAR:
-	    *(char *)rp->slot = Get_Integer (*p); break;
-	case T_PIXEL:
-	    *(unsigned long *)rp->slot = Get_Pixel (*p); break;
-	case T_BACKGROUND:
-	    if (EQ(*p, Sym_None))
-		*(Pixmap *)rp->slot = None;
-	    else if (EQ(*p, Sym_Parent_Relative))
-		*(Pixmap *)rp->slot = ParentRelative;
-	    else
-		*(Pixmap *)rp->slot = Get_Pixmap (*p);
-	    break;
-	case T_BORDER:
-	    if (EQ(*p, Sym_Copy_From_Parent)) {
-		*(Pixmap *)rp->slot = CopyFromParent;
-		break;
-	    }
-	    /* fall through */
-	case T_PIXMAP:
-	    *(Pixmap *)rp->slot = Get_Pixmap (*p); break;
-	case T_BOOL:
-	    Check_Type (*p, T_Boolean);
-	    *(Bool *)rp->slot = (Bool)(FIXNUM(*p));
-	    break;
-	case T_FONT:
-	    *(Font *)rp->slot = Get_Font (*p);
-	    break;
-	case T_COLORMAP:
-	    *(Colormap *)rp->slot = Get_Colormap (*p); break;
-	case T_CURSOR:
-	    *(Cursor *)rp->slot = Get_Cursor (*p);
-	    break;
-	case T_WINDOW:
-	    break;
-	case T_MASK:
-	    *(long *)rp->slot = Symbols_To_Bits (*p, 1, rp->syms);
-	    break;
-	case T_SYM:
-	    *(int *)rp->slot = (int)Symbols_To_Bits (*p, 0, rp->syms);
-	    break;
-	default:
-	    Panic ("vector->record");
-	}
-	mask |= rp->mask;
+        ++p;
+        if (rp->type == T_NONE || Nullp (*p))
+            continue;
+        switch (rp->type) {
+        case T_INT:
+            *(int *)rp->slot = Get_Integer (*p); break;
+        case T_SHORT:
+            *(short *)rp->slot = Get_Integer (*p); break;
+        case T_CHAR:
+            *(char *)rp->slot = Get_Integer (*p); break;
+        case T_PIXEL:
+            *(unsigned long *)rp->slot = Get_Pixel (*p); break;
+        case T_BACKGROUND:
+            if (EQ(*p, Sym_None))
+                *(Pixmap *)rp->slot = None;
+            else if (EQ(*p, Sym_Parent_Relative))
+                *(Pixmap *)rp->slot = ParentRelative;
+            else
+                *(Pixmap *)rp->slot = Get_Pixmap (*p);
+            break;
+        case T_BORDER:
+            if (EQ(*p, Sym_Copy_From_Parent)) {
+                *(Pixmap *)rp->slot = CopyFromParent;
+                break;
+            }
+            /* fall through */
+        case T_PIXMAP:
+            *(Pixmap *)rp->slot = Get_Pixmap (*p); break;
+        case T_BOOL:
+            Check_Type (*p, T_Boolean);
+            *(Bool *)rp->slot = (Bool)(FIXNUM(*p));
+            break;
+        case T_FONT:
+            *(Font *)rp->slot = Get_Font (*p);
+            break;
+        case T_COLORMAP:
+            *(Colormap *)rp->slot = Get_Colormap (*p); break;
+        case T_CURSOR:
+            *(Cursor *)rp->slot = Get_Cursor (*p);
+            break;
+        case T_WINDOW:
+            break;
+        case T_MASK:
+            *(long *)rp->slot = Symbols_To_Bits (*p, 1, rp->syms);
+            break;
+        case T_SYM:
+            *(int *)rp->slot = (int)Symbols_To_Bits (*p, 0, rp->syms);
+            break;
+        default:
+            Panic ("vector->record");
+        }
+        mask |= rp->mask;
     }
     return mask;
 }
 
-Object Record_To_Vector (rp, len, sym, dpy, flags) Object sym;
-	register RECORD *rp; Display *dpy; unsigned long flags; {
-    register i;
+Object Record_To_Vector (register RECORD *rp, int len, Object sym,
+                         Display *dpy, unsigned long flags) {
+    register int i;
     Object v, x;
     GC_Node2;
 
@@ -346,53 +376,53 @@ Object Record_To_Vector (rp, len, sym, dpy, flags) Object sym;
     v = Make_Vector (len, Null);
     VECTOR(v)->data[0] = sym;
     for (i = 1; rp->slot; i++, rp++) {
-	if (rp->type == T_NONE)
-	    continue;
-	if (rp->mask && !(flags & rp->mask))
-	    continue;
-	x = Null;
-	switch (rp->type) {
-	case T_INT:
-	    x = Make_Integer (*(int *)rp->slot); break;
-	case T_SHORT:
-	    x = Make_Integer (*(short *)rp->slot); break;
-	case T_CHAR:
-	    x = Make_Integer (*(char *)rp->slot); break;
-	case T_PIXEL:
-	    x = Make_Pixel (*(unsigned long *)rp->slot); break;
-	case T_PIXMAP:
-	    if (*(unsigned long *)rp->slot == ~0L)
-		x = Sym_None;
-	    else
-		x = Make_Pixmap_Foreign (dpy, *(Pixmap *)rp->slot);
-	    break;
-	case T_FONT:
-	    if (*(unsigned long *)rp->slot == ~0L)
-		x = Sym_None;
-	    else {
-		register XFontStruct *info;
-		Disable_Interrupts;
-		info = XQueryFont (dpy, *(Font *)rp->slot);
-		Enable_Interrupts;
-		x = Make_Font_Foreign (dpy, False, *(Font *)rp->slot, info);
-	    }
-	    break;
-	case T_BOOL:
-	    x = *(Bool *)rp->slot ? True : False; break;
-	case T_COLORMAP:
-	    x = Make_Colormap (0, dpy, *(Colormap *)rp->slot); break;
-	case T_WINDOW:
-	    x = Make_Window (0, dpy, *(Window *)rp->slot); break;
-	case T_MASK:
-	    x = Bits_To_Symbols (*(long *)rp->slot, 1, rp->syms);
-	    break;
-	case T_SYM:
-	    x = Bits_To_Symbols ((unsigned long)*(int *)rp->slot, 0, rp->syms);
-	    break;
-	default:
-	    Panic ("record->vector");
-	}
-	VECTOR(v)->data[i] = x;
+        if (rp->type == T_NONE)
+            continue;
+        if (rp->mask && !(flags & rp->mask))
+            continue;
+        x = Null;
+        switch (rp->type) {
+        case T_INT:
+            x = Make_Integer (*(int *)rp->slot); break;
+        case T_SHORT:
+            x = Make_Integer (*(short *)rp->slot); break;
+        case T_CHAR:
+            x = Make_Integer (*(char *)rp->slot); break;
+        case T_PIXEL:
+            x = Make_Pixel (*(unsigned long *)rp->slot); break;
+        case T_PIXMAP:
+            if (*(unsigned long *)rp->slot == ~0UL)
+                x = Sym_None;
+            else
+                x = Make_Pixmap_Foreign (dpy, *(Pixmap *)rp->slot);
+            break;
+        case T_FONT:
+            if (*(unsigned long *)rp->slot == ~0UL)
+                x = Sym_None;
+            else {
+                register XFontStruct *info;
+                Disable_Interrupts;
+                info = XQueryFont (dpy, *(Font *)rp->slot);
+                Enable_Interrupts;
+                x = Make_Font_Foreign (dpy, False, *(Font *)rp->slot, info);
+            }
+            break;
+        case T_BOOL:
+            x = *(Bool *)rp->slot ? True : False; break;
+        case T_COLORMAP:
+            x = Make_Colormap (0, dpy, *(Colormap *)rp->slot); break;
+        case T_WINDOW:
+            x = Make_Window (0, dpy, *(Window *)rp->slot); break;
+        case T_MASK:
+            x = Bits_To_Symbols (*(long *)rp->slot, 1, rp->syms);
+            break;
+        case T_SYM:
+            x = Bits_To_Symbols ((unsigned long)*(int *)rp->slot, 0, rp->syms);
+            break;
+        default:
+            Panic ("record->vector");
+        }
+        VECTOR(v)->data[i] = x;
     }
     GC_Unlink;
     return v;
@@ -763,41 +793,40 @@ SYMDESCR Error_Syms[] = {
     { 0, 0 }
 };
 
-static Init_Record (rec, size, name, var) RECORD *rec; char *name;
-	Object *var; {
+static void Init_Record (RECORD *rec, int size, char *name, Object *var) {
     Object list, tail, cell;
-    register i;
+    register int i;
     char buf[128];
     GC_Node2;
 
     GC_Link2 (list, tail);
     for (list = tail = Null, i = 1; i < size; tail = cell, i++, rec++) {
-	cell = Intern (rec->name);
-	cell = Cons (cell, Make_Integer (i));
-	cell = Cons (cell, Null);
-	if (Nullp (list))
-	    list = cell;
-	else
-	    P_Set_Cdr (tail, cell);
+        cell = Intern (rec->name);
+        cell = Cons (cell, Make_Integer (i));
+        cell = Cons (cell, Null);
+        if (Nullp (list))
+            list = cell;
+        else
+            P_Set_Cdr (tail, cell);
     }
     sprintf (buf, "%s-slots", name);
     Define_Variable (var, buf, list);
     GC_Unlink;
 }
 
-elk_init_xlib_type () {
+void elk_init_xlib_type () {
     Init_Record (Set_Attr_Rec, Set_Attr_Size, "set-window-attributes",
-	&Set_Attr_Slots);
+        &Set_Attr_Slots);
     Init_Record (Conf_Rec, Conf_Size, "window-configuration", &Conf_Slots);
     Init_Record (GC_Rec, GC_Size, "gcontext", &GC_Slots);
     Init_Record (Geometry_Rec, Geometry_Size, "geometry", &Geometry_Slots);
     Init_Record (Win_Attr_Rec, Win_Attr_Size, "get-window-attributes",
-	&Win_Attr_Slots);
+        &Win_Attr_Slots);
     Init_Record (Font_Info_Rec, Font_Info_Size, "font-info", &Font_Info_Slots);
     Init_Record (Char_Info_Rec, Char_Info_Size, "char-info", &Char_Info_Slots);
     Init_Record (Wm_Hints_Rec, Wm_Hints_Size, "wm-hints", &Wm_Hints_Slots);
     Init_Record (Size_Hints_Rec, Size_Hints_Size, "size-hints",
-	&Size_Hints_Slots);
+        &Size_Hints_Slots);
     Define_Symbol (&Sym_Parent_Relative, "parent-relative");
     Define_Symbol (&Sym_Copy_From_Parent, "copy-from-parent");
 }
