@@ -1,3 +1,35 @@
+/* xlib.h
+ *
+ * $Id: xlib.h 243 2003-12-05 12:38:37Z sam $
+ *
+ * Copyright 1990, 1991, 1992, 1993, 1994, 1995, Oliver Laumann, Berlin
+ * Copyright 2002, 2003 Sam Hocevar <sam@zoy.org>, Paris
+ *
+ * This software was derived from Elk 1.2, which was Copyright 1987, 1988,
+ * 1989, Nixdorf Computer AG and TELES GmbH, Berlin (Elk 1.2 has been written
+ * by Oliver Laumann for TELES Telematic Services, Berlin, in a joint project
+ * between TELES and Nixdorf Microprocessor Engineering, Berlin).
+ *
+ * Oliver Laumann, TELES GmbH, Nixdorf Computer AG and Sam Hocevar, as co-
+ * owners or individual owners of copyright in this software, grant to any
+ * person or company a worldwide, royalty free, license to
+ *
+ *    i) copy this software,
+ *   ii) prepare derivative works based on this software,
+ *  iii) distribute copies of this software or derivative works,
+ *   iv) perform this software, or
+ *    v) display this software,
+ *
+ * provided that this notice is not removed and that neither Oliver Laumann
+ * nor Teles nor Nixdorf are deemed to have made any representations as to
+ * the suitability of this software for any purpose nor are held responsible
+ * for any defects of this software.
+ *
+ * THERE IS ABSOLUTELY NO WARRANTY FOR THIS SOFTWARE.
+ */
+
+#include "config.h"
+
 #include <X11/X.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -136,7 +168,7 @@ extern Pixmap Get_Pixmap P_((Object));
 extern Time Get_Time P_((Object));
 extern Window Get_Window P_((Object));
 extern XColor *Get_Color P_((Object));
-extern unsigned long Get_Pixel P_((Object));
+extern unsigned long int Get_Pixel P_((Object));
 extern void Destroy_Event_Args P_((Object));
 extern int Encode_Event P_((Object));
 extern int Match_X_Obj P_((ELLIPSIS));
@@ -150,7 +182,7 @@ extern Object Make_Display P_((int, Display*));
 extern Object Make_Font P_((Display*, Object, Font, XFontStruct*));
 extern Object Make_Font_Foreign P_((Display*, Object, Font, XFontStruct*));
 extern Object Make_Gc P_((int, Display*, GC));
-extern Object Make_Pixel P_((unsigned long));
+extern Object Make_Pixel P_((unsigned long int));
 extern Object Make_Pixmap P_((Display*, Pixmap));
 extern Object Make_Pixmap_Foreign P_((Display*, Pixmap));
 extern Object Make_Window P_((int, Display*, Window));
@@ -163,8 +195,32 @@ extern Object P_Free_Gc P_((Object));
 extern Object P_Free_Pixmap P_((Object));
 extern Object P_Window_Unique_Id P_((Object));
 extern Object Record_To_Vector
-    P_((RECORD*, int, Object, Display*, unsigned long));
-extern unsigned long Vector_To_Record P_((Object, int, Object, RECORD*));
+    P_((RECORD*, int, Object, Display*, unsigned long int));
+extern unsigned long int Vector_To_Record
+    P_((Object, unsigned int, Object, RECORD*));
+
+extern void elk_init_xlib_client P_(());
+extern void elk_init_xlib_color P_(());
+extern void elk_init_xlib_colormap P_(());
+extern void elk_init_xlib_cursor P_(());
+extern void elk_init_xlib_display P_(());
+extern void elk_init_xlib_error P_(());
+extern void elk_init_xlib_event P_(());
+extern void elk_init_xlib_extension P_(());
+extern void elk_init_xlib_font P_(());
+extern void elk_init_xlib_gcontext P_(());
+extern void elk_init_xlib_grab P_(());
+extern void elk_init_xlib_graphics P_(());
+extern void elk_init_xlib_key P_(());
+extern void elk_init_xlib_objects P_(());
+extern void elk_init_xlib_pixel P_(());
+extern void elk_init_xlib_pixmap P_(());
+extern void elk_init_xlib_property P_(());
+extern void elk_init_xlib_text P_(());
+extern void elk_init_xlib_type P_(());
+extern void elk_init_xlib_util P_(());
+extern void elk_init_xlib_window P_(());
+extern void elk_init_xlib_wm P_(());
 
 C_LINKAGE_END
 
@@ -178,7 +234,7 @@ extern XCharStruct CI;
 extern XWMHints WMH;
 extern XSizeHints SZH;
 
-extern Set_Attr_Size, Conf_Size, GC_Size, Geometry_Size, Win_Attr_Size,
+extern int Set_Attr_Size, Conf_Size, GC_Size, Geometry_Size, Win_Attr_Size,
     Font_Info_Size, Char_Info_Size, Wm_Hints_Size, Size_Hints_Size;
 extern RECORD Set_Attr_Rec[], Conf_Rec[], GC_Rec[], Geometry_Rec[],
     Win_Attr_Rec[], Font_Info_Rec[], Char_Info_Rec[], Wm_Hints_Rec[],
@@ -212,54 +268,54 @@ extern Object Sym_None, Sym_Now, Sym_Char_Info, Sym_Conf;
  *
  *   int T_Pixmap;
  *
- *   static Object P_Pixmapp (x) Object x; {
+ *   static Object P_Pixmapp (Object x) {
  *        return TYPE(x) == T_Pixmap ? True : False;
  *   }
  */
 #define Generic_Predicate(type) int conc(T_,type);\
 \
-static Object conc3(P_,type,p) (x) Object x; {\
+static Object conc3(P_,type,p) (Object x) {\
     return TYPE(x) == conc(T_,type) ? True : False;\
 }
 
 /* Generic_Equal (Pixmap, PIXMAP, pm) generates:
  *
- *    static Pixmap_Equal (x, y) Object x, y; {
+ *    static Pixmap_Equal (Object x, Object y) {
  *        return PIXMAP(x)->pm == PIXMAP(y)->field
  *            && !PIXMAP(x)->free && !PIXMAP(y)->free;
  *    }
  */
-#define Generic_Equal(type,cast,field) static conc(type,_Equal) (x, y)\
-	Object x, y; {\
+#define Generic_Equal(type,cast,field) static int conc(type,_Equal)\
+            (Object x, Object y) {\
     return cast(x)->field == cast(y)->field\
-	&& !cast(x)->free && !cast(y)->free;\
+        && !cast(x)->free && !cast(y)->free;\
 }
 
 /* Same as above, but doesn't check for ->free:
  */
-#define Generic_Simple_Equal(type,cast,field) static conc(type,_Equal) (x, y)\
-	Object x, y; {\
+#define Generic_Simple_Equal(type,cast,field) static int conc(type,_Equal)\
+            (Object x, Object y) {\
     return cast(x)->field == cast(y)->field;\
 }
 
 /* Same as above, but also checks ->dpy
  */
-#define Generic_Equal_Dpy(type,cast,field) static conc(type,_Equal)\
-	    (x, y)\
-	Object x, y; {\
+#define Generic_Equal_Dpy(type,cast,field) static int conc(type,_Equal)\
+            (Object x, Object y) {\
     return cast(x)->field == cast(y)->field && cast(x)->dpy == cast(y)->dpy\
-	&& !cast(x)->free && !cast(y)->free;\
+        && !cast(x)->free && !cast(y)->free;\
 }
 
 /* Generic_Print (Pixmap, "#[pixmap %u]", PIXMAP(x)->pm) generates:
  *
- *    static Pixmap_Print (x, port, raw, depth, len) Object x, port; {
+ *    static Pixmap_Print (Object x, Object port, int raw, int depth, int len) {
  *        Printf (port, "#[pixmap %u]", PIXMAP(x)->pm);
  *    }
  */
-#define Generic_Print(type,fmt,how) static conc(type,_Print)\
-	(x, port, raw, depth, len) Object x, port; {\
+#define Generic_Print(type,fmt,how) static int conc(type,_Print)\
+            (Object x, Object port, int raw, int depth, int len) {\
     Printf (port, fmt, (unsigned)how);\
+    return 0;\
 }
 
 /* Generic_Define (Pixmap, "pixmap", "pixmap?") generates:
@@ -270,18 +326,18 @@ static Object conc3(P_,type,p) (x) Object x; {\
  */
 #define Generic_Define(type,name,pred) conc(T_,type) =\
     Define_Type (0, name, NOFUNC, sizeof (struct conc(S_,type)),\
-	conc(type,_Equal), conc(type,_Equal), conc(type,_Print), NOFUNC);\
+        conc(type,_Equal), conc(type,_Equal), conc(type,_Print), NOFUNC);\
     Define_Primitive (conc3(P_,type,p), pred, 1, 1, EVAL);
 
 /* Generic_Get_Display (Pixmap, PIXMAP) generates:
  *
- *    static Object P_Pixmap_Display (x) Object x; {
+ *    static Object P_Pixmap_Display (Object x) {
  *        Check_Type (x, T_Pixmap);
  *        return Make_Display (PIXMAP(x)->dpy);
  *    }
  */
 #define Generic_Get_Display(type,cast) static Object conc3(P_,type,_Display)\
-	(x) Object x; {\
+            (Object x) {\
     Check_Type (x, conc(T_,type));\
     return Make_Display (0, cast(x)->dpy);\
 }
